@@ -49,7 +49,9 @@ sub new {
 
 	# lookup hash for elements by name attribute
 	$self->{names} = {};
-	
+
+	$self->{pagings} = {};
+
 	bless $self;
 }
 
@@ -192,6 +194,31 @@ sub list_add {
 	return $listref;
 }
 
+=head2 paging_add PAGING
+
+=cut
+
+sub paging_add {
+    my ($self, $new_pagingref) = @_;
+    my ($name, $class, $pagingref);
+
+	$name = $new_pagingref->{paging}->{name};
+
+	$pagingref = $self->{pagings}->{$name} = {elements => $new_pagingref->{paging}->{elements}, list => $new_pagingref->{paging}->{list}};
+
+    # loop through paging elements
+    for my $element (values %{$new_pagingref->{paging}->{elements}}) {
+        $class = $element->{class} || $element->{name};
+        push @{$self->{classes}->{$class}}, {%{$element}, element_type => $element->{type}, type => 'element', list => $new_pagingref->{paging}->{list}, paging => $name};
+    }
+    
+	$class = $new_pagingref->{paging}->{class} || $name;
+
+	push @{$self->{classes}->{$class}}, {%{$new_pagingref->{paging}}, type => 'paging'};
+
+    return $pagingref;
+}
+
 =head2 form_add FORM
 
 Add form specified by hash reference FORM.
@@ -277,7 +304,11 @@ sub value_add {
 		# include implies hooking resulting value
 		$new_valueref->{value}->{op} = 'hook';
 	}
-	
+	elsif (exists $new_valueref->{value}->{field}
+           && $new_valueref->{value}->{field} =~ /\./) {
+        $new_valueref->{value}->{field} = [split /\./, $new_valueref->{value}->{field}];
+    }
+
 	$valueref = $self->{values}->{$new_valueref->{value}->{name}} = {};
 	
 	if ($id = $new_valueref->{value}->{id}) {
@@ -509,9 +540,14 @@ Returns paging for list NAME.
 	
 sub list_paging {
 	my ($self, $list_name) = @_;
+    my ($name, $paging_ref);
 
 	if (exists $self->{lists}->{$list_name}) {
-		return $self->{lists}->{$list_name}->{paging};
+        while (($name, $paging_ref) = each %{$self->{pagings}}) {
+            if ($paging_ref->{list} eq $list_name) {
+                return $paging_ref;
+            }
+        }
 	}	
 }
 
@@ -526,7 +562,7 @@ the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Template-F
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010-2012 Stefan Hornburg (Racke).
+Copyright 2010-2013 Stefan Hornburg (Racke).
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
